@@ -361,7 +361,11 @@ model_specs <-
         "bm_svr",
         "bm_mars",
         "bm_cubist",
-        "bm_pls_pcr"
+        "bm_pls_pcr",
+        "bm_prophet",
+        "bm_ets",
+        "bm_auto_arima",
+        "bm_xgb"
         )
 
     if (!all(learner %in% .available_models))
@@ -468,8 +472,29 @@ setMethod("show",
 #' model is the one that has the lowest loss prediction from
 #' the meta models. Defaults to FALSE;
 #'
+#' @slot all_models Logical. If true, at each prediction time,
+#' all base models are picked to make a prediction. The
+#' models are weighted according to their predicted loss
+#' and the \code{aggregation} function. Defaults to FALSE;
+#'
+#' @slot aggregation Type of aggregation used to combine the
+#' predictions of the base models. The options are:
+#' \describe{
+#'  \item{softmax}{default}
+#'  \item{erfc}{the complementary Gaussian error function}
+#'  \item{linear}{a linear scaling}
+#' }
+#'
+#' @slot sequential_reweight Besides ensemble heterogeneity we encourage diversity
+#' explicitly during the aggregation of the output of experts.
+#' This is achieved by taking into account not only predictions
+#' of performance produced by the arbiters, but also the
+#' correlation among experts in a recent window of observations.
+#'
 #' @slot recent_series the most recent \code{lambda} observations.
 #'
+#' @slot out_of_bag oob
+#'
 #' @references Cerqueira, Vitor; Torgo, Luis; Pinto, Fabio;
 #' and Soares, Carlos. "Arbitrated Ensemble for Time Series
 #' Forecasting" to appear at: Joint European Conference on Machine Learning and
@@ -510,7 +535,7 @@ setMethod("show",
 #' train <- train[1:300, ] # toy size for checks
 #'
 #' model <- ADE(target ~., train, specs)
-#' 
+#'
 #'
 #' @export
 setClass("ADE",
@@ -521,7 +546,11 @@ setClass("ADE",
                    lambda = "numeric",
                    omega = "OptionalNumeric",
                    select_best = "logical",
-                   recent_series = "data.frame")
+                   all_models = "logical",
+                   aggregation = "character",
+                   sequential_reweight = "logical",
+                   recent_series = "data.frame",
+                   out_of_bag = "OptionalList")
 )
 
 #' Arbitrated Dynamic Ensemble
@@ -563,6 +592,25 @@ setClass("ADE",
 #' model is the one that has the lowest loss prediction from
 #' the meta models. Defaults to FALSE;
 #'
+#' @param all_models Logical. If true, at each prediction time,
+#' all base models are picked to make a prediction. The
+#' models are weighted according to their predicted loss
+#' and the \code{aggregation} function. Defaults to FALSE;
+#'
+#' @param aggregation Type of aggregation used to combine the
+#' predictions of the base models. The options are:
+#' \describe{
+#'  \item{softmax}{default}
+#'  \item{erfc}{the complementary Gaussian error function}
+#'  \item{linear}{a linear scaling}
+#' }
+#'
+#' @param sequential_reweight Besides ensemble heterogeneity we encourage diversity
+#' explicitly during the aggregation of the output of experts.
+#' This is achieved by taking into account not only predictions
+#' of performance produced by the arbiters, but also the
+#' correlation among experts in a recent window of observations.
+#'
 #' @references Cerqueira, Vitor; Torgo, Luis; Pinto, Fabio;
 #' and Soares, Carlos. "Arbitrated Ensemble for Time Series
 #' Forecasting" to appear at: Joint European Conference on Machine Learning and
@@ -611,7 +659,10 @@ setClass("ADE",
            specs,
            lambda = 50,
            omega = .5,
-           select_best = FALSE) {
+           select_best = FALSE,
+           all_models = FALSE,
+           aggregation = "linear",
+           sequential_reweight = FALSE) {
 
     if (select_best && is.numeric(omega))
       warning(
@@ -620,6 +671,17 @@ setClass("ADE",
         \"omega\" parameter will be ignored."
         )
 
+    if (all_models && is.numeric(omega))
+      warning(
+        "ADE setup with both selection of all learners and
+        a committee (\"omega\" parameter).
+        \"omega\" parameter will be ignored."
+        )
+
+    if (select_best && all_models)
+      stop("Choose either \"select_best\" or \"all_models\" option.",
+           call. = FALSE)
+
     if (!is.null(omega))
       if (omega >= 1 | omega <= 0)
         stop("\"omega\" parameter must be a double between 0 and 1",
@@ -634,8 +696,8 @@ setClass("ADE",
         form = form,
         train = data,
         specs = specs,
-        lambda = lambda
-      )
+        lambda = lambda,
+        lfun = ae)
 
     methods::new(
       "ADE",
@@ -646,7 +708,11 @@ setClass("ADE",
       lambda = lambda,
       omega = omega,
       select_best = select_best,
-      recent_series = M$recent_series
+      all_models = all_models,
+      aggregation = aggregation,
+      sequential_reweight = sequential_reweight,
+      recent_series = M$recent_series,
+      out_of_bag = M$OOB
     )
   }
 
@@ -686,7 +752,6 @@ setClass("ade_hat",
 )
 
 
-
 #' Predictions by an ADE ensemble
 #'
 #' Predictions produced by a \code{\link{ADE-class}} object.
@@ -776,6 +841,11 @@ ade_hat <- function(y_hat, Y_hat, Y_committee, E_hat) {
 #' to their performance in the last \emph{lambda} observations.
 #' Defaults to .5 according to empirical experiments;
 #'
+#' @slot select_best Logical. If true, at each prediction time,
+#' a single base model is picked to make a prediction. The picked
+#' model is the one that has the lowest loss prediction from
+#' the meta models. Defaults to FALSE;
+#'
 #' @slot recent_series the most recent \code{lambda} observations.
 #'
 #' @references Cerqueira, Vitor; Torgo, Luis; Oliveira, Mariana,
@@ -813,6 +883,7 @@ setClass("DETS",
                    specs = "model_specs",
                    lambda = "OptionalNumeric",
                    omega = "OptionalNumeric",
+                   select_best = "logical",
                    recent_series = "data.frame")
 )
 
@@ -862,6 +933,11 @@ setClass("DETS",
 #' to their performance in the last \emph{lambda} observations.
 #' Defaults to .5 according to empirical experiments;
 #'
+#' @param select_best Logical. If true, at each prediction time,
+#' a single base model is picked to make a prediction. The picked
+#' model is the one that has the lowest loss prediction from
+#' the meta models. Defaults to FALSE;
+#'
 #' @references Cerqueira, Vitor; Torgo, Luis; Oliveira, Mariana,
 #' and Bernhard Pfahringer. "Dynamic and Heterogeneous Ensembles
 #' for Time Series Forecasting." Data Science and Advanced
@@ -896,7 +972,8 @@ setClass("DETS",
            data,
            specs,
            lambda = 50,
-           omega = .5) {
+           omega = .5,
+           select_best = FALSE) {
 
     M <- build_base_ensemble(form, data, specs)
 
@@ -909,6 +986,7 @@ setClass("DETS",
       specs = specs,
       lambda = lambda,
       omega = omega,
+      select_best = select_best,
       recent_series = recent_lambda_k
     )
   }
@@ -1011,7 +1089,7 @@ dets_hat <- function(y_hat, Y_hat, Y_committee, W) {
 #'
 #' next_vals_dets <- forecast(model, h = 2)
 #' next_vals_ade <- forecast(model2, h = 2)
-#' 
+#'
 #'
 #' @export
 setGeneric("forecast",
